@@ -22,8 +22,12 @@
  */
 
 #include "transcode.h"
+#include "libtc/libtc.h"
+#include "tcinfo.h"
+
 #include "ioaux.h"
-#include "avilib.h"
+#include "avilib/avilib.h"
+#include "tc.h"
 
 #ifdef HAVE_LZO
 
@@ -56,7 +60,7 @@ void decode_lzo(decode_t *decode)
      */
 
     if (lzo_init() != LZO_E_OK) {
-      fprintf(stderr, "[%s] lzo_init() failed\n", MOD_NAME);
+      tc_log_error(__FILE__, "lzo_init() failed");
       goto decoder_error;
     }
 
@@ -65,7 +69,7 @@ void decode_lzo(decode_t *decode)
     inbuf = (lzo_bytep) lzo_malloc(BUFFER_SIZE);
 
     if (wrkmem == NULL || out == NULL) {
-      fprintf(stderr, "[%s] out of memory\n", MOD_NAME);
+      tc_log_error(__FILE__, "out of memory");
       goto decoder_error;
     }
 
@@ -74,29 +78,29 @@ void decode_lzo(decode_t *decode)
     verbose = decode->verbose;
 
     for (;;) {
-	if ( (ss=p_read (decode->fd_in, (char *)&h, sizeof(h))) != sizeof(h)) {
-	    //fprintf(stderr," (%s) failed to read frame size: EOF. expected (%ld) got (%d)\n", __FILE__, 4L, ss);
+	if ( (ss=tc_pread (decode->fd_in, (uint8_t *)&h, sizeof(h))) != sizeof(h)) {
+	    //tc_log_msg(__FILE__, "failed to read frame size: EOF. expected (%ld) got (%d)", 4L, ss);
 	    goto decoder_out;
 	}
 
 	// check magic
 	if (h.magic != TC_CODEC_LZO2) {
-	    fprintf(stderr," (%s) Wrong stream magic: expected (0x%x) got (0x%x)\n", __FILE__, TC_CODEC_LZO2, h.magic);
+	    tc_log_error(__FILE__, "Wrong stream magic: expected (0x%x) got (0x%x)", TC_CODEC_LZO2, h.magic);
 	    goto decoder_error;
 	}
 
 	//str2long(bb, &bytes);
 	bytes = h.size;
 
-	if (verbose & TC_DEBUG) 
-	    fprintf (stderr, "got bytes (%ld)\n", bytes); 
-	if ( (ss=p_read (decode->fd_in, inbuf, bytes))!=bytes) {
-	    fprintf(stderr," (%s) failed to read frame: expected (%ld) got (%lu)\n", __FILE__, bytes, (unsigned long)ss);
+	if (verbose & TC_DEBUG)
+	    tc_log_msg(__FILE__, "got bytes (%ld)", bytes);
+	if ( (ss=tc_pread (decode->fd_in, inbuf, bytes))!=bytes) {
+	    tc_log_error(__FILE__, "failed to read frame: expected (%ld) got (%lu)", bytes, (unsigned long)ss);
 	    goto decoder_error;
 	}
 
 	if (h.flags & TC_LZO_NOT_COMPRESSIBLE) {
-	  tc_memcpy(out, inbuf, bytes);
+	  ac_memcpy(out, inbuf, bytes);
 	  out_len = bytes;
 	  r = LZO_E_OK;
 	} else {
@@ -105,17 +109,17 @@ void decode_lzo(decode_t *decode)
 
 	if (r == LZO_E_OK) {
 	    if(verbose & TC_DEBUG)
-		fprintf(stderr, "decompressed %lu bytes into %lu bytes\n",
-				    (long) bytes, (long) out_len);
+		tc_log_msg(__FILE__, "decompressed %lu bytes into %lu bytes",
+			   (long) bytes, (long) out_len);
 	} else {
 
 	    /* this should NEVER happen */
-	    fprintf(stderr, "[%s] internal error - decompression failed: %d\n", MOD_NAME, r);
+	    tc_log_error(__FILE__, "internal error - decompression failed: %d", r);
 	    goto decoder_error;
 	}
 
-	if ( (ss = p_write (decode->fd_out, out, out_len)) != out_len) {
-	    fprintf(stderr," (%s) failed to write frame: expected (%ld) wrote (%lu)\n", __FILE__, bytes, (unsigned long)ss);
+	if ( (ss = tc_pwrite (decode->fd_out, out, out_len)) != out_len) {
+	    tc_log_error(__FILE__, "failed to write frame: expected (%ld) wrote (%lu)", bytes, (unsigned long)ss);
 	    goto decoder_error;
 	}
     }
@@ -132,7 +136,7 @@ decoder_error:
 
 void decode_lzo(decode_t *decode)
 {
-    fprintf(stderr, "No support for LZO configured -- exiting\n");
+    tc_log_error(__FILE__, "No support for LZO configured -- exiting");
     import_exit(1);
 }
 
