@@ -23,44 +23,46 @@
  *
  */
 
+#include "transcode.h"
+
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <string.h>
-#include <inttypes.h>
 
-#include "xio.h"
-
-#include "config.h"
-#include "transcode.h"
+#include "libtc/xio.h"
 
 #include "aud_scan.h"
 
 #define EXE "aviindex"
 
-void version()
+/* AVI_info is no longer in avilib */
+void AVI_info(avi_t *avifile);
+
+void version(void)
 {
   printf("%s (%s v%s) (C) 2003-2004 Tilmann Bitterberg,"
-                        " 2004-2008 Transcode Team\n",
+                        " 2004-2009 Transcode Team\n",
                       EXE, PACKAGE, VERSION);
 }
 
 
-void usage(int status)
+static void usage(int status)
 {
   version();
   printf("\nUsage: %s [options]\n", EXE);
-  printf("\t -o file   output file\n");
-  printf("\t -i file   input file\n");
-  printf("\t -f        force the use of the existing index\n");
-  printf("\t           only to use when avi > 2GB, because\n");
-  printf("\t           the default is to -n with big files\n");
-  printf("\t -n        read index in \"smart\" mode: don't use the existing index\n");
-  printf("\t -x        don't use the existing index to generate the keyframes\n");
-  printf("\t           this flag forces -n\n");
-  printf("\t -v        print version\n");
+  printf("    -o file   output file\n");
+  printf("    -i file   input file\n");
+  printf("    -f        force the use of the existing index\n");
+  printf("              only to use when avi > 2GB, because\n");
+  printf("              the default is to -n with big files\n");
+  printf("    -n        read index in \"smart\" mode: don't use the existing index\n");
+  printf("    -x        don't use the existing index to generate the keyframes\n");
+  printf("              this flag forces -n\n");
+  printf("    -v        print version\n");
   exit(status);
 }
 
@@ -75,11 +77,11 @@ static unsigned long str2ulong(unsigned char *str)
 static inline uint32_t SWAP(uint32_t a) {
 #ifdef WORDS_BIGENDIAN
     return ( (a<<24&0xff000000) | (a<< 8&0x00ff0000) |
-	     (a>> 8&0x0000ff00) | (a>>24&0x000000ff)); 
+	     (a>> 8&0x0000ff00) | (a>>24&0x000000ff));
 #else
     return a;
 #endif
-} 
+}
 
 static inline int avi_stream_id(unsigned int id){
   unsigned char *p=(unsigned char *)&id;
@@ -136,7 +138,7 @@ typedef struct {
     uint32_t dwChunkLength;
 } AVIINDEXENTRY;
 
-int aviidx1_to_mpidx1(char *in_file, FILE *out_fd)
+static int aviidx1_to_mpidx1(char *in_file, FILE *out_fd)
 {
     char *data;
     FILE *in;
@@ -189,7 +191,8 @@ int aviidx1_to_mpidx1(char *in_file, FILE *out_fd)
     fclose(out_fd);
     return 0;
 }
-int mpidx1_to_aviidx1(char *in_file, FILE *out_fd)
+
+static int mpidx1_to_aviidx1(char *in_file, FILE *out_fd)
 {
     char head[10];
     char *data;
@@ -216,29 +219,29 @@ int mpidx1_to_aviidx1(char *in_file, FILE *out_fd)
 	uint32_t ckid;
 	idx = &((AVIINDEXENTRY *)data)[i];
 	ckid = SWAP(idx->ckid);
-	fprintf(out_fd, 
-		"%.4s %d %d %d %d %d %d 0\n", 
-		(char *)&ckid, 
+	fprintf(out_fd,
+		"%.4s %d %d %d %d %d %d 0\n",
+		(char *)&ckid,
 		avi_stream_nr(idx->ckid),
-		i, 
+		i,
 		streams[avi_stream_id(idx->ckid)],
-		idx->dwChunkOffset, 
-		idx->dwChunkLength, 
+		idx->dwChunkOffset,
+		idx->dwChunkLength,
 		idx->dwFlags?1:0);
 
 	streams[avi_stream_id(idx->ckid)]++;
     }
 
-    
+
     free(data);
     fclose (in);
     fclose(out_fd);
-    
+
     return 0;
 }
 
 // data is only 8 bytes long
-int AVI_read_data_fast(avi_t *AVI, char *buf, off_t *pos, off_t *len, off_t *key, char *data)
+static int AVI_read_data_fast(avi_t *AVI, char *buf, off_t *pos, off_t *len, off_t *key, char *data)
 {
 
 /*
@@ -267,16 +270,22 @@ int AVI_read_data_fast(avi_t *AVI, char *buf, off_t *pos, off_t *len, off_t *key
 
       n = PAD_EVEN(str2ulong(data+4));
 
-      if(strncasecmp(data,"LIST",4) == 0) {
-	  continue;
-      }
-
-      if(strncasecmp(data,"movi",4) == 0 ||
-         strncasecmp(data,"rec ",4) == 0) {
-	  xio_lseek(AVI->fdes,-4,SEEK_CUR);
-	  continue;
+      if(strncasecmp(data,"LIST",4) == 0 ||
+	 strncasecmp(data,"RIFF",4) == 0) { // prevents skipping extended RIFF chunks
+	  if( xio_read(AVI->fdes,data,4) != 4 ) return 0;
+	  n -= 4;
+	  // put here tags of lists that need to be looked into
+	  if(strncasecmp(data,"movi",4) == 0 ||
+	     strncasecmp(data,"rec ",4) == 0 ||
+	     strncasecmp(data,"AVI ",4) == 0 ||
+	     strncasecmp(data,"AVIX",4) == 0) {
+	    // xio_lseek(AVI->fdes,-4,SEEK_CUR);
+	    continue; // proceed to look into it
+	  } // otherwise seek over it later on
       }
 
+      // the following list of comparisons should not include list tags;
+      // these should all go in the list above
       if(strncasecmp(data,"IDX1",4) == 0)
       {
 	 // deal with it to extract keyframe info
@@ -391,7 +400,7 @@ int AVI_read_data_fast(avi_t *AVI, char *buf, off_t *pos, off_t *len, off_t *key
    }
 }
 
-int is_key(unsigned char *data, long size, char *codec) 
+static int is_key(unsigned char *data, long size, char *codec)
 {
     if (strncasecmp(codec, "div3", 4) == 0) {
 
@@ -414,7 +423,7 @@ int is_key(unsigned char *data, long size, char *codec)
                         else                            return 0;
                 }
         }
-        
+
         return result;
 
     }
@@ -457,7 +466,7 @@ int main(int argc, char *argv[])
   char tag[8];
   char *data;
   int vid_chunks=0, aud_chunks[AVI_MAX_TRACKS];
-  off_t pos, len, key, index_pos=0, index_len=0,size=0;
+  off_t pos, len, key=0, index_pos=0, index_len=0,size=0;
   struct stat st;
   char *codec;
   int idx_type=0;
@@ -467,7 +476,7 @@ int main(int argc, char *argv[])
   ftype_t ftype;
   FILE *idxfile;
 
-  tc_memcpy_init(0, -1);
+  ac_init(AC_ALL);
 
   if(argc==1) usage(EXIT_FAILURE);
 
@@ -476,51 +485,51 @@ int main(int argc, char *argv[])
     aud_ms[i] = 0;
   }
 
-  while ((ch = getopt(argc, argv, "a:vi:o:nf?h")) != -1)
+  while ((ch = getopt(argc, argv, "a:vi:o:nxf?h")) != -1)
     {
 
 	switch (ch) {
-	
+
 	case 'i':
 
 	     if(optarg[0]=='-') usage(EXIT_FAILURE);
 	    in_file=optarg;
-	
+
 	    break;
-	
+
 	case 'a':
-	
+
 	  if(optarg[0]=='-') usage(EXIT_FAILURE);
 	  track_num = atoi(optarg);
-	
+
 	  if(track_num<0) usage(EXIT_FAILURE);
-	
+
 	  break;
 
 	case 'o':
 
 	    if(optarg[0]=='-') usage(EXIT_FAILURE);
 	    out_file=optarg;
-	
+
 	    break;
-	
+
 	case 'n':
 
 	    open_without_index=1;
-	
+
 	    break;
 
 	case 'x':
 
 	    open_without_index=1;
 	    index_keyframes=1;
-	
+
 	    break;
 
 	case 'f':
 	    force_with_index=1;
 	    break;
-	
+
 	case 'v':
 	    version();
 	    exit(0);
@@ -553,9 +562,9 @@ int main(int argc, char *argv[])
 
   switch (ftype) {
       case RIFF:    fprintf(stderr, "[%s] Seems to be an AVI file.\n", EXE); break;
-      case AVIIDX1: fprintf(stderr, "[%s] Converting a transcode to an mplayer index file.\n", EXE); 
+      case AVIIDX1: fprintf(stderr, "[%s] Converting a transcode to an mplayer index file.\n", EXE);
 		    return aviidx1_to_mpidx1(in_file, out_fd);
-      case MPIDX1:  fprintf(stderr, "[%s] Converting an mplayer to a transcode index file.\n", EXE); 
+      case MPIDX1:  fprintf(stderr, "[%s] Converting an mplayer to a transcode index file.\n", EXE);
 		    return mpidx1_to_aviidx1(in_file, out_fd);
       default:      fprintf(stderr, "[%s] Unrecognized format\n", EXE); return (1);
   }
@@ -572,7 +581,7 @@ int main(int argc, char *argv[])
   if (size > (off_t)AVI_MAX_LEN/2)
       if (!force_with_index) open_without_index = 1;
 
-  if (open_without_index) 
+  if (open_without_index)
       if (index_keyframes) fprintf(stderr, "[%s] Open \"%s\" without index and don't use index for keyframes info\n",EXE, in_file);
       else fprintf(stderr, "[%s] Open \"%s\" without index but use index (if any) for keyframes info\n",EXE, in_file);
   else
@@ -627,7 +636,7 @@ int main(int argc, char *argv[])
       }
 
       switch (ret) {
-	case 1: tc_memcpy(tag, fcclen, 4);
+	case 1: ac_memcpy(tag, fcclen, 4);
 		print_ms = vid_ms = (avifile1->video_pos)*1000.0/fps;
 		chunk = avifile1->video_pos;
 		key = is_key(data, len, codec);
@@ -636,11 +645,11 @@ int main(int argc, char *argv[])
 	case 4: case 5:
 	case 6: case 7:
 	case 8:
-	case 9: tc_memcpy(tag, fcclen, 4);
+	case 9: ac_memcpy(tag, fcclen, 4);
 		print_ms = aud_ms[audtr];
 		chunk = avifile1->track[audtr].audio_posc;
 		break;
-	case 10: snprintf(tag, sizeof(tag), "idx1");
+	case 10: tc_snprintf(tag, sizeof(tag), "idx1");
 		 index_pos = pos;
 		 index_len = len;
 		 print_ms = 0.0;
@@ -656,7 +665,10 @@ int main(int argc, char *argv[])
 
       //if (index_pos != pos)
       // tag, chunk_nr
-      fprintf(out_fd, "%.4s %d %ld %ld %lld %lld %lld %.2f\n", tag, ret, i, chunk-1, pos, len, key, print_ms);
+      fprintf(out_fd, "%.4s %d %ld %ld %lld %lld %lld %.2f\n",
+              tag, ret, i, chunk-1,
+              (long long)pos, (long long)len, (long long)key,
+              print_ms);
       i++;
 
       // don't update the counter every chunk
@@ -685,12 +697,12 @@ int main(int argc, char *argv[])
     xio_lseek(avifile1->fdes, index_pos+8, SEEK_SET);
     while (len<index_len) {
 	xio_read(avifile1->fdes, tag, 8);
-	
+
 	// if its a keyframe and is a video chunk
 	if (str2ulong(tag+4) && tag[1] == '0') {
 	    int typen, keyn;
 	    long chunkn, chunkptypen;
-	    off_t posn, lenn;
+	    long long posn, lenn;
 	    char tagn[5];
 	    double msn=0.0;
 
@@ -709,16 +721,16 @@ int main(int argc, char *argv[])
 	    posn = ftell(out_fd);
 	    fgets(data, 100, out_fd);
 	    fseek(out_fd, posn, SEEK_SET);
-	    sscanf(data, "%s %d %ld %ld %lld %lld %d %lf", 
+	    sscanf(data, "%s %d %ld %ld %lld %lld %d %lf",
 		      tagn, &typen, &chunkn, &chunkptypen, &posn, &lenn, &keyn, &msn);
-	    fprintf(out_fd, "%s %d %ld %ld %lld %lld %d %.2f", 
+	    fprintf(out_fd, "%s %d %ld %ld %lld %lld %d %.2f",
 		      tagn, typen, chunkn, chunkptypen, posn, lenn, 1, msn);
 	}
 
 	xio_lseek(avifile1->fdes, 8, SEEK_CUR);
 	len += 16;
     }
-    
+
 
 
   } else { // with index
@@ -732,7 +744,8 @@ int main(int argc, char *argv[])
 
     AVI_info(avifile1);
 
-    if(avifile1->idx)
+    // idx1 contains only info for first chunk of opendml AVI
+    if(avifile1->idx && !avifile1->is_opendml)
     {
       off_t pos, len;
 
@@ -761,60 +774,121 @@ int main(int argc, char *argv[])
          }
       }
       /* idx_type remains 0 if neither of the two tests above succeeds */
-   }
 
-   ioff = idx_type == 1 ? 0 : avifile1->movi_start-4;
-   //fprintf(stderr, "index type (%d), ioff (%ld)\n", idx_type, (long)ioff);
-    i=0;
 
-    //printf("nr idx: %d\n", avifile1->n_idx);
-    while (i<avifile1->n_idx) {
-      tc_memcpy(tag, avifile1->idx[i], 4);
-      // tag
-      fprintf(out_fd, "%c%c%c%c", 
-	  avifile1->idx[i][0], avifile1->idx[i][1],
-	  avifile1->idx[i][2], avifile1->idx[i][3]);
+      ioff = idx_type == 1 ? 0 : avifile1->movi_start-4;
+    //fprintf(stderr, "index type (%d), ioff (%ld)\n", idx_type, (long)ioff);
+      i=0;
 
-      // type, absolute chunk number
-      fprintf(out_fd, " %c %ld", avifile1->idx[i][1]+1, i);
-      
+      //printf("nr idx: %d\n", avifile1->n_idx);
+      while (i<avifile1->n_idx) {
+	ac_memcpy(tag, avifile1->idx[i], 4);
+	// tag
+	fprintf(out_fd, "%c%c%c%c",
+	    avifile1->idx[i][0], avifile1->idx[i][1],
+	    avifile1->idx[i][2], avifile1->idx[i][3]);
 
-      switch (avifile1->idx[i][1]) {
-	case '0':
-	  fprintf(out_fd, " %d", vid_chunks);
-	  vid_chunks++;
-	  break;
-	case '1': case '2':
-	case '3': case '4':
-	case '5': case '6':
-	case '7': case '8':
-	  // uhoh
-	  ret = avifile1->idx[i][1]-'0';
-	  fprintf(out_fd, " %d", aud_chunks[ret]);
-	  aud_chunks[ret]++;
-	  break;
-	default:
-	  fprintf(out_fd, " %d", -1);
-	  break;
+	// type, absolute chunk number
+	fprintf(out_fd, " %c %ld", avifile1->idx[i][1]+1, i);
+
+
+	switch (avifile1->idx[i][1]) {
+	  case '0':
+	    fprintf(out_fd, " %d", vid_chunks);
+	    vid_chunks++;
+	    break;
+	  case '1': case '2':
+	  case '3': case '4':
+	  case '5': case '6':
+	  case '7': case '8':
+	    // uhoh
+	    ret = avifile1->idx[i][1]-'0';
+	    fprintf(out_fd, " %d", aud_chunks[ret]);
+	    aud_chunks[ret]++;
+	    break;
+	  default:
+	    fprintf(out_fd, " %d", -1);
+	    break;
+	}
+
+	pos = str2ulong(avifile1->idx[i]+ 8);
+	pos += ioff;
+	// pos
+	fprintf(out_fd, " %llu", (unsigned long long)pos);
+	// len
+	fprintf(out_fd, " %lu", str2ulong(avifile1->idx[i]+12));
+	// flags (keyframe?);
+	fprintf(out_fd, " %d", (str2ulong(avifile1->idx[i]+ 4))?1:0);
+
+	// ms (not available here)
+	fprintf(out_fd, " %.2f", 0.0);
+
+	fprintf(out_fd, "\n");
+
+	i++;
       }
-
-      pos = str2ulong(avifile1->idx[i]+ 8);
-      pos += ioff;
-      // pos
-      fprintf(out_fd, " %llu", pos);
-      // len
-      fprintf(out_fd, " %lu", str2ulong(avifile1->idx[i]+12));
-      // flags (keyframe?);
-      fprintf(out_fd, " %d", (str2ulong(avifile1->idx[i]+ 4))?1:0);
-
-      // ms (not available here)
-      fprintf(out_fd, " %.2f", 0.0);
-
-      fprintf(out_fd, "\n");
-
-      i++;
     }
 
+    else
+    { // try to extract from the index that AVILIB built,
+      // possibly from OpenDML superindex
+
+      long aud_entry [ AVI_MAX_TRACKS ] = { 0 };
+      long vid_entry = 0;
+      char* tagp;
+
+      off_t pos, len = 0;
+      i = chunk = 0;
+
+
+      while (1) {
+	ret = pos = 0;
+	int j = 0;
+
+	if(vid_entry < avifile1->video_frames) {
+	  pos = avifile1->video_index[vid_entry].pos;
+	  len = avifile1->video_index[vid_entry].len;
+	  key = (avifile1->video_index[vid_entry].key) & 16 ? 1 : 0;
+	  chunk = vid_entry;
+	  ret = 1;
+	}
+	for(j = 0; j < AVI_audio_tracks(avifile1); ++j) {
+	  if(aud_entry[j] < avifile1->track[j].audio_chunks) {
+	    if(!ret || avifile1->track[j].audio_index[aud_entry[j]].pos < pos) {
+	      pos = avifile1->track[j].audio_index[aud_entry[j]].pos;
+	      len = avifile1->track[j].audio_index[aud_entry[j]].len;
+	      key = 0;
+	      chunk = aud_entry[j];
+	      ret = j + 2;
+	    }
+	  }
+	}
+
+	if(!ret) // end of all index streams
+	  break;
+
+	if (ret == 1)
+	{
+	  ++vid_entry;
+	  tagp = avifile1->video_tag;
+	}
+	else
+	{
+	  aud_entry[ret-2]++;
+	  tagp = avifile1->track[ret-2].audio_tag;
+	}
+
+	// index points to data in chunk, but chunk offset is needed here
+	pos -= 8;
+	fprintf(out_fd, "%.4s %d %ld %ld %lld %lld %lld %.2f\n",
+                    tagp, ret, i, chunk,
+                    (long long)pos, (long long)len, (long long)key,
+                    0.0);
+	i++;
+
+      }
+
+    }
 
   }
 
@@ -826,3 +900,15 @@ aviout:
 
   return(0);
 }
+
+/*************************************************************************/
+
+/*
+ * Local variables:
+ *   c-file-style: "stroustrup"
+ *   c-file-offsets: ((case-label . *) (statement-case-intro . *))
+ *   indent-tabs-mode: nil
+ * End:
+ *
+ * vim: expandtab shiftwidth=4:
+ */
